<?php

namespace SuperRhino\Utils\KeyStore;

use SuperRhino\Utils\Cache;
use SuperRhino\Utils\Facade;

class KeyStoreValue implements KeyStoreValueInterface {

    const CACHE_PREFIX = 'ksv';

    /**
     * @var KeyStoreValueInterface
     */
    private $dbAdapter;

    public function __construct() {
        $keyStoreDbAdapter = Facade::getContainer()->get(KeyStoreDbAdapterInterface::class);
        $this->dbAdapter = $keyStoreDbAdapter->getValueAdapter();
    }

    public function getValue(string $key, string $default = ''): string {
        return Cache::getCallback($this->getCacheKey($key), function () use ($key, $default) {
            return $this->dbAdapter->getValue($key, $default);
        }, 60 * 60);
    }

    public function setValue(string $key, string $value): bool {
        if ($result = $this->dbAdapter->setValue($key, $value)) {
            Cache::delete($this->getCacheKey($key));
        }
        return $result;
    }

    public function delValue(string $key): bool {
        $this->dbAdapter->delValue($key);
        Cache::delete($this->getCacheKey($key));
        return true;
    }

    private function getCacheKey(string $key): string {
        return self::CACHE_PREFIX . ':' . $key;
    }
}